@@ -115,13 +115,13 @@ class InfluxDBClient(object):
         also be passed to this function.
 
         Examples:
-            >>> cli = InfluxDBClient.from_DSN('influxdb://username:password@\
+            >> cli = InfluxDBClient.from_DSN('influxdb://username:password@\
             ... localhost:8086/databasename', timeout=5)
-            >>> type(cli)
+            >> type(cli)
             <class 'influxdb.client.InfluxDBClient'>
-            >>> cli = InfluxDBClient.from_DSN('udp+influxdb://username:pass@\
+            >> cli = InfluxDBClient.from_DSN('udp+influxdb://username:pass@\
             ... localhost:8086/databasename', timeout=5, udp_port=159)
-            >>> print('{0._baseurl} - {0.use_udp} {0.udp_port}'.format(cli))
+            >> print('{0._baseurl} - {0.use_udp} {0.udp_port}'.format(cli))
             http://localhost:8086 - True 159
 
         :param dsn: data source name
@@ -242,7 +242,8 @@ class InfluxDBClient(object):
                     timeout=self._timeout
                 )
                 break
-            except requests.exceptions.ConnectionError as e:
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.Timeout) as e:
                 if i < 2:
                     continue
                 else:
